@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -49,87 +49,79 @@
 /* should we include monitors in the device list? Set at SDL_Init time */
 static SDL_bool include_monitors = SDL_FALSE;
 
-
 #if (PA_API_VERSION < 12)
 /** Return non-zero if the passed state is one of the connected states */
-static SDL_INLINE int PA_CONTEXT_IS_GOOD(pa_context_state_t x) {
-    return
-        x == PA_CONTEXT_CONNECTING ||
-        x == PA_CONTEXT_AUTHORIZING ||
-        x == PA_CONTEXT_SETTING_NAME ||
-        x == PA_CONTEXT_READY;
+static SDL_INLINE int PA_CONTEXT_IS_GOOD(pa_context_state_t x)
+{
+    return x == PA_CONTEXT_CONNECTING || x == PA_CONTEXT_AUTHORIZING || x == PA_CONTEXT_SETTING_NAME || x == PA_CONTEXT_READY;
 }
 /** Return non-zero if the passed state is one of the connected states */
-static SDL_INLINE int PA_STREAM_IS_GOOD(pa_stream_state_t x) {
-    return
-        x == PA_STREAM_CREATING ||
-        x == PA_STREAM_READY;
+static SDL_INLINE int PA_STREAM_IS_GOOD(pa_stream_state_t x)
+{
+    return x == PA_STREAM_CREATING || x == PA_STREAM_READY;
 }
 #endif /* pulseaudio <= 0.9.10 */
 
-
-static const char *(*PULSEAUDIO_pa_get_library_version) (void);
-static pa_channel_map *(*PULSEAUDIO_pa_channel_map_init_auto) (
+static const char *(*PULSEAUDIO_pa_get_library_version)(void);
+static pa_channel_map *(*PULSEAUDIO_pa_channel_map_init_auto)(
     pa_channel_map *, unsigned, pa_channel_map_def_t);
-static const char * (*PULSEAUDIO_pa_strerror) (int);
-static pa_mainloop * (*PULSEAUDIO_pa_mainloop_new) (void);
-static pa_mainloop_api * (*PULSEAUDIO_pa_mainloop_get_api) (pa_mainloop *);
-static int (*PULSEAUDIO_pa_mainloop_iterate) (pa_mainloop *, int, int *);
-static int (*PULSEAUDIO_pa_mainloop_run) (pa_mainloop *, int *);
-static void (*PULSEAUDIO_pa_mainloop_quit) (pa_mainloop *, int);
-static void (*PULSEAUDIO_pa_mainloop_free) (pa_mainloop *);
+static const char *(*PULSEAUDIO_pa_strerror)(int);
+static pa_mainloop *(*PULSEAUDIO_pa_mainloop_new)(void);
+static pa_mainloop_api *(*PULSEAUDIO_pa_mainloop_get_api)(pa_mainloop *);
+static int (*PULSEAUDIO_pa_mainloop_iterate)(pa_mainloop *, int, int *);
+static int (*PULSEAUDIO_pa_mainloop_run)(pa_mainloop *, int *);
+static void (*PULSEAUDIO_pa_mainloop_quit)(pa_mainloop *, int);
+static void (*PULSEAUDIO_pa_mainloop_free)(pa_mainloop *);
 
-static pa_operation_state_t (*PULSEAUDIO_pa_operation_get_state) (
+static pa_operation_state_t (*PULSEAUDIO_pa_operation_get_state)(
     const pa_operation *);
-static void (*PULSEAUDIO_pa_operation_cancel) (pa_operation *);
-static void (*PULSEAUDIO_pa_operation_unref) (pa_operation *);
+static void (*PULSEAUDIO_pa_operation_cancel)(pa_operation *);
+static void (*PULSEAUDIO_pa_operation_unref)(pa_operation *);
 
-static pa_context * (*PULSEAUDIO_pa_context_new) (pa_mainloop_api *,
-    const char *);
-static int (*PULSEAUDIO_pa_context_connect) (pa_context *, const char *,
-    pa_context_flags_t, const pa_spawn_api *);
-static pa_operation * (*PULSEAUDIO_pa_context_get_sink_info_list) (pa_context *, pa_sink_info_cb_t, void *);
-static pa_operation * (*PULSEAUDIO_pa_context_get_source_info_list) (pa_context *, pa_source_info_cb_t, void *);
-static pa_operation * (*PULSEAUDIO_pa_context_get_sink_info_by_index) (pa_context *, uint32_t, pa_sink_info_cb_t, void *);
-static pa_operation * (*PULSEAUDIO_pa_context_get_source_info_by_index) (pa_context *, uint32_t, pa_source_info_cb_t, void *);
-static pa_context_state_t (*PULSEAUDIO_pa_context_get_state) (const pa_context *);
-static pa_operation * (*PULSEAUDIO_pa_context_subscribe) (pa_context *, pa_subscription_mask_t, pa_context_success_cb_t, void *);
-static void (*PULSEAUDIO_pa_context_set_subscribe_callback) (pa_context *, pa_context_subscribe_cb_t, void *);
-static void (*PULSEAUDIO_pa_context_disconnect) (pa_context *);
-static void (*PULSEAUDIO_pa_context_unref) (pa_context *);
+static pa_context *(*PULSEAUDIO_pa_context_new)(pa_mainloop_api *,
+                                                const char *);
+static int (*PULSEAUDIO_pa_context_connect)(pa_context *, const char *,
+                                            pa_context_flags_t, const pa_spawn_api *);
+static pa_operation *(*PULSEAUDIO_pa_context_get_sink_info_list)(pa_context *, pa_sink_info_cb_t, void *);
+static pa_operation *(*PULSEAUDIO_pa_context_get_source_info_list)(pa_context *, pa_source_info_cb_t, void *);
+static pa_operation *(*PULSEAUDIO_pa_context_get_sink_info_by_index)(pa_context *, uint32_t, pa_sink_info_cb_t, void *);
+static pa_operation *(*PULSEAUDIO_pa_context_get_source_info_by_index)(pa_context *, uint32_t, pa_source_info_cb_t, void *);
+static pa_context_state_t (*PULSEAUDIO_pa_context_get_state)(const pa_context *);
+static pa_operation *(*PULSEAUDIO_pa_context_subscribe)(pa_context *, pa_subscription_mask_t, pa_context_success_cb_t, void *);
+static void (*PULSEAUDIO_pa_context_set_subscribe_callback)(pa_context *, pa_context_subscribe_cb_t, void *);
+static void (*PULSEAUDIO_pa_context_disconnect)(pa_context *);
+static void (*PULSEAUDIO_pa_context_unref)(pa_context *);
 
-static pa_stream * (*PULSEAUDIO_pa_stream_new) (pa_context *, const char *,
-    const pa_sample_spec *, const pa_channel_map *);
-static int (*PULSEAUDIO_pa_stream_connect_playback) (pa_stream *, const char *,
-    const pa_buffer_attr *, pa_stream_flags_t, const pa_cvolume *, pa_stream *);
-static int (*PULSEAUDIO_pa_stream_connect_record) (pa_stream *, const char *,
-    const pa_buffer_attr *, pa_stream_flags_t);
-static pa_stream_state_t (*PULSEAUDIO_pa_stream_get_state) (const pa_stream *);
-static size_t (*PULSEAUDIO_pa_stream_writable_size) (const pa_stream *);
-static size_t (*PULSEAUDIO_pa_stream_readable_size) (const pa_stream *);
-static int (*PULSEAUDIO_pa_stream_begin_write) (pa_stream *, void **, size_t*);
-static int (*PULSEAUDIO_pa_stream_cancel_write) (pa_stream *);
-static int (*PULSEAUDIO_pa_stream_write) (pa_stream *, const void *, size_t,
-    pa_free_cb_t, int64_t, pa_seek_mode_t);
-static pa_operation * (*PULSEAUDIO_pa_stream_drain) (pa_stream *,
-    pa_stream_success_cb_t, void *);
-static int (*PULSEAUDIO_pa_stream_peek) (pa_stream *, const void **, size_t *);
-static int (*PULSEAUDIO_pa_stream_drop) (pa_stream *);
-static pa_operation * (*PULSEAUDIO_pa_stream_flush) (pa_stream *,
-    pa_stream_success_cb_t, void *);
-static int (*PULSEAUDIO_pa_stream_disconnect) (pa_stream *);
-static void (*PULSEAUDIO_pa_stream_unref) (pa_stream *);
+static pa_stream *(*PULSEAUDIO_pa_stream_new)(pa_context *, const char *,
+                                              const pa_sample_spec *, const pa_channel_map *);
+static int (*PULSEAUDIO_pa_stream_connect_playback)(pa_stream *, const char *,
+                                                    const pa_buffer_attr *, pa_stream_flags_t, const pa_cvolume *, pa_stream *);
+static int (*PULSEAUDIO_pa_stream_connect_record)(pa_stream *, const char *,
+                                                  const pa_buffer_attr *, pa_stream_flags_t);
+static pa_stream_state_t (*PULSEAUDIO_pa_stream_get_state)(const pa_stream *);
+static size_t (*PULSEAUDIO_pa_stream_writable_size)(const pa_stream *);
+static size_t (*PULSEAUDIO_pa_stream_readable_size)(const pa_stream *);
+static int (*PULSEAUDIO_pa_stream_write)(pa_stream *, const void *, size_t,
+                                         pa_free_cb_t, int64_t, pa_seek_mode_t);
+static pa_operation *(*PULSEAUDIO_pa_stream_drain)(pa_stream *,
+                                                   pa_stream_success_cb_t, void *);
+static int (*PULSEAUDIO_pa_stream_peek)(pa_stream *, const void **, size_t *);
+static int (*PULSEAUDIO_pa_stream_drop)(pa_stream *);
+static pa_operation *(*PULSEAUDIO_pa_stream_flush)(pa_stream *,
+                                                   pa_stream_success_cb_t, void *);
+static int (*PULSEAUDIO_pa_stream_disconnect)(pa_stream *);
+static void (*PULSEAUDIO_pa_stream_unref)(pa_stream *);
+static void (*PULSEAUDIO_pa_stream_set_write_callback)(pa_stream *, pa_stream_request_cb_t, void *);
+static pa_operation *(*PULSEAUDIO_pa_context_get_server_info)(pa_context *, pa_server_info_cb_t, void *);
 
 static int load_pulseaudio_syms(void);
 
-
 #ifdef SDL_AUDIO_DRIVER_PULSEAUDIO_DYNAMIC
 
 static const char *pulseaudio_library = SDL_AUDIO_DRIVER_PULSEAUDIO_DYNAMIC;
 static void *pulseaudio_handle = NULL;
 
-static int
-load_pulseaudio_sym(const char *fn, void **addr)
+static int load_pulseaudio_sym(const char *fn, void **addr)
 {
     *addr = SDL_LoadFunction(pulseaudio_handle, fn);
     if (*addr == NULL) {
@@ -141,11 +133,11 @@ load_pulseaudio_sym(const char *fn, void **addr)
 }
 
 /* cast funcs to char* first, to please GCC's strict aliasing rules. */
-#define SDL_PULSEAUDIO_SYM(x) \
-    if (!load_pulseaudio_sym(#x, (void **) (char *) &PULSEAUDIO_##x)) return -1
+#define SDL_PULSEAUDIO_SYM(x)                                       \
+    if (!load_pulseaudio_sym(#x, (void **)(char *)&PULSEAUDIO_##x)) \
+    return -1
 
-static void
-UnloadPulseAudioLibrary(void)
+static void UnloadPulseAudioLibrary(void)
 {
     if (pulseaudio_handle != NULL) {
         SDL_UnloadObject(pulseaudio_handle);
@@ -153,8 +145,7 @@ UnloadPulseAudioLibrary(void)
     }
 }
 
-static int
-LoadPulseAudioLibrary(void)
+static int LoadPulseAudioLibrary(void)
 {
     int retval = 0;
     if (pulseaudio_handle == NULL) {
@@ -176,13 +167,11 @@ LoadPulseAudioLibrary(void)
 
 #define SDL_PULSEAUDIO_SYM(x) PULSEAUDIO_##x = x
 
-static void
-UnloadPulseAudioLibrary(void)
+static void UnloadPulseAudioLibrary(void)
 {
 }
 
-static int
-LoadPulseAudioLibrary(void)
+static int LoadPulseAudioLibrary(void)
 {
     load_pulseaudio_syms();
     return 0;
@@ -190,9 +179,7 @@ LoadPulseAudioLibrary(void)
 
 #endif /* SDL_AUDIO_DRIVER_PULSEAUDIO_DYNAMIC */
 
-
-static int
-load_pulseaudio_syms(void)
+static int load_pulseaudio_syms(void)
 {
     SDL_PULSEAUDIO_SYM(pa_get_library_version);
     SDL_PULSEAUDIO_SYM(pa_mainloop_new);
@@ -222,8 +209,6 @@ load_pulseaudio_syms(void)
     SDL_PULSEAUDIO_SYM(pa_stream_writable_size);
     SDL_PULSEAUDIO_SYM(pa_stream_readable_size);
     SDL_PULSEAUDIO_SYM(pa_stream_write);
-    SDL_PULSEAUDIO_SYM(pa_stream_begin_write);
-    SDL_PULSEAUDIO_SYM(pa_stream_cancel_write);
     SDL_PULSEAUDIO_SYM(pa_stream_drain);
     SDL_PULSEAUDIO_SYM(pa_stream_disconnect);
     SDL_PULSEAUDIO_SYM(pa_stream_peek);
@@ -232,18 +217,18 @@ load_pulseaudio_syms(void)
     SDL_PULSEAUDIO_SYM(pa_stream_unref);
     SDL_PULSEAUDIO_SYM(pa_channel_map_init_auto);
     SDL_PULSEAUDIO_SYM(pa_strerror);
+    SDL_PULSEAUDIO_SYM(pa_stream_set_write_callback);
+    SDL_PULSEAUDIO_SYM(pa_context_get_server_info);
     return 0;
 }
 
-static SDL_INLINE int
-squashVersion(const int major, const int minor, const int patch)
+static SDL_INLINE int squashVersion(const int major, const int minor, const int patch)
 {
     return ((major & 0xFF) << 16) | ((minor & 0xFF) << 8) | (patch & 0xFF);
 }
 
 /* Workaround for older pulse: pa_context_new() must have non-NULL appname */
-static const char *
-getAppName(void)
+static const char *getAppName(void)
 {
     const char *retval = SDL_GetHint(SDL_HINT_AUDIO_DEVICE_APP_NAME);
     if (retval && *retval) {
@@ -254,12 +239,12 @@ getAppName(void)
         return retval;
     } else {
         const char *verstr = PULSEAUDIO_pa_get_library_version();
-        retval = "SDL Application";  /* the "oh well" default. */
+        retval = "SDL Application"; /* the "oh well" default. */
         if (verstr != NULL) {
             int maj, min, patch;
             if (SDL_sscanf(verstr, "%d.%d.%d", &maj, &min, &patch) == 3) {
                 if (squashVersion(maj, min, patch) >= squashVersion(0, 9, 15)) {
-                    retval = NULL;  /* 0.9.15+ handles NULL correctly. */
+                    retval = NULL; /* 0.9.15+ handles NULL correctly. */
                 }
             }
         }
@@ -267,8 +252,7 @@ getAppName(void)
     return retval;
 }
 
-static void
-WaitForPulseOperation(pa_mainloop *mainloop, pa_operation *o)
+static void WaitForPulseOperation(pa_mainloop *mainloop, pa_operation *o)
 {
     /* This checks for NO errors currently. Either fix that, check results elsewhere, or do things you don't care about. */
     if (mainloop && o) {
@@ -280,8 +264,7 @@ WaitForPulseOperation(pa_mainloop *mainloop, pa_operation *o)
     }
 }
 
-static void
-DisconnectFromPulseServer(pa_mainloop *mainloop, pa_context *context)
+static void DisconnectFromPulseServer(pa_mainloop *mainloop, pa_context *context)
 {
     if (context) {
         PULSEAUDIO_pa_context_disconnect(context);
@@ -292,8 +275,7 @@ DisconnectFromPulseServer(pa_mainloop *mainloop, pa_context *context)
     }
 }
 
-static int
-ConnectToPulseServer_Internal(pa_mainloop **_mainloop, pa_context **_context)
+static int ConnectToPulseServer_Internal(pa_mainloop **_mainloop, pa_context **_context)
 {
     pa_mainloop *mainloop = NULL;
     pa_context *context = NULL;
@@ -309,10 +291,10 @@ ConnectToPulseServer_Internal(pa_mainloop **_mainloop, pa_context **_context)
     }
 
     mainloop_api = PULSEAUDIO_pa_mainloop_get_api(mainloop);
-    SDL_assert(mainloop_api);  /* this never fails, right? */
+    SDL_assert(mainloop_api); /* this never fails, right? */
 
     context = PULSEAUDIO_pa_context_new(mainloop_api, getAppName());
-    if (!context) {
+    if (context == NULL) {
         PULSEAUDIO_pa_mainloop_free(mainloop);
         return SDL_SetError("pa_context_new() failed");
     }
@@ -341,11 +323,10 @@ ConnectToPulseServer_Internal(pa_mainloop **_mainloop, pa_context **_context)
     *_context = context;
     *_mainloop = mainloop;
 
-    return 0;  /* connected and ready! */
+    return 0; /* connected and ready! */
 }
 
-static int
-ConnectToPulseServer(pa_mainloop **_mainloop, pa_context **_context)
+static int ConnectToPulseServer(pa_mainloop **_mainloop, pa_context **_context)
 {
     const int retval = ConnectToPulseServer_Internal(_mainloop, _context);
     if (retval < 0) {
@@ -354,61 +335,60 @@ ConnectToPulseServer(pa_mainloop **_mainloop, pa_context **_context)
     return retval;
 }
 
-
 /* This function waits until it is possible to write a full sound buffer */
-static void
-PULSEAUDIO_WaitDevice(_THIS)
+static void PULSEAUDIO_WaitDevice(_THIS)
+{
+    /* this is a no-op; we wait in PULSEAUDIO_PlayDevice now. */
+}
+
+static void WriteCallback(pa_stream *p, size_t nbytes, void *userdata)
+{
+    struct SDL_PrivateAudioData *h = (struct SDL_PrivateAudioData *)userdata;
+    /*printf("PULSEAUDIO WRITE CALLBACK! nbytes=%u\n", (unsigned int) nbytes);*/
+    h->bytes_requested += nbytes;
+}
+
+static void PULSEAUDIO_PlayDevice(_THIS)
 {
     struct SDL_PrivateAudioData *h = this->hidden;
+    int available = h->mixlen;
+    int written = 0;
+    int cpy;
 
-    while (SDL_AtomicGet(&this->enabled)) {
+    /*printf("PULSEAUDIO PLAYDEVICE START! mixlen=%d\n", available);*/
+
+    while (SDL_AtomicGet(&this->enabled) && (available > 0)) {
+        cpy = SDL_min(h->bytes_requested, available);
+        if (cpy) {
+            if (PULSEAUDIO_pa_stream_write(h->stream, h->mixbuf + written, cpy, NULL, 0LL, PA_SEEK_RELATIVE) < 0) {
+                SDL_OpenedAudioDeviceDisconnected(this);
+                return;
+            }
+            /*printf("PULSEAUDIO FEED! nbytes=%u\n", (unsigned int) cpy);*/
+            h->bytes_requested -= cpy;
+            written += cpy;
+            available -= cpy;
+        }
+
+        /* let WriteCallback fire if necessary. */
+        /*printf("PULSEAUDIO ITERATE!\n");*/
         if (PULSEAUDIO_pa_context_get_state(h->context) != PA_CONTEXT_READY ||
             PULSEAUDIO_pa_stream_get_state(h->stream) != PA_STREAM_READY ||
             PULSEAUDIO_pa_mainloop_iterate(h->mainloop, 1, NULL) < 0) {
             SDL_OpenedAudioDeviceDisconnected(this);
             return;
         }
-        if (PULSEAUDIO_pa_stream_writable_size(h->stream) >= (h->mixlen/8)) {
-            return;
-        }
     }
+
+    /*printf("PULSEAUDIO PLAYDEVICE END! written=%d\n", written);*/
 }
 
-static void
-PULSEAUDIO_PlayDevice(_THIS)
+static Uint8 *PULSEAUDIO_GetDeviceBuf(_THIS)
 {
-    /* Write the audio data */
-    struct SDL_PrivateAudioData *h = this->hidden;
-    if (SDL_AtomicGet(&this->enabled)) {
-        if (PULSEAUDIO_pa_stream_write(h->stream, h->pabuf, h->mixlen, NULL, 0LL, PA_SEEK_RELATIVE) < 0) {
-            SDL_OpenedAudioDeviceDisconnected(this);
-        }
-    }
+    return this->hidden->mixbuf;
 }
 
-static Uint8 *
-PULSEAUDIO_GetDeviceBuf(_THIS)
-{
-    struct SDL_PrivateAudioData *h = this->hidden;
-    size_t nbytes = h->mixlen;
-    int ret;
-
-    ret = PULSEAUDIO_pa_stream_begin_write(h->stream, &h->pabuf, &nbytes);
-
-    if (ret != 0) {
-        /* fall back it intermediate buffer */
-        h->pabuf = h->mixbuf;
-    } else if (nbytes < h->mixlen) {
-        PULSEAUDIO_pa_stream_cancel_write(h->stream);
-        h->pabuf = h->mixbuf;
-    }
-
-    return (Uint8 *)h->pabuf;
-}
-
-
-static int
-PULSEAUDIO_CaptureFromDevice(_THIS, void *buffer, int buflen)
+static int PULSEAUDIO_CaptureFromDevice(_THIS, void *buffer, int buflen)
 {
     struct SDL_PrivateAudioData *h = this->hidden;
     const void *data = NULL;
@@ -423,40 +403,40 @@ PULSEAUDIO_CaptureFromDevice(_THIS, void *buffer, int buflen)
             h->capturelen -= cpy;
             if (h->capturelen == 0) {
                 h->capturebuf = NULL;
-                PULSEAUDIO_pa_stream_drop(h->stream);  /* done with this fragment. */
+                PULSEAUDIO_pa_stream_drop(h->stream); /* done with this fragment. */
             }
-            return cpy;  /* new data, return it. */
+            return cpy; /* new data, return it. */
         }
 
         if (PULSEAUDIO_pa_context_get_state(h->context) != PA_CONTEXT_READY ||
             PULSEAUDIO_pa_stream_get_state(h->stream) != PA_STREAM_READY ||
             PULSEAUDIO_pa_mainloop_iterate(h->mainloop, 1, NULL) < 0) {
             SDL_OpenedAudioDeviceDisconnected(this);
-            return -1;  /* uhoh, pulse failed! */
+            return -1; /* uhoh, pulse failed! */
         }
 
         if (PULSEAUDIO_pa_stream_readable_size(h->stream) == 0) {
-            continue;  /* no data available yet. */
+            continue; /* no data available yet. */
         }
 
         /* a new fragment is available! */
         PULSEAUDIO_pa_stream_peek(h->stream, &data, &nbytes);
         SDL_assert(nbytes > 0);
-        if (data == NULL) {  /* NULL==buffer had a hole. Ignore that. */
-            PULSEAUDIO_pa_stream_drop(h->stream);  /* drop this fragment. */
+        /* If data == NULL, then the buffer had a hole, ignore that */
+        if (data == NULL) {
+            PULSEAUDIO_pa_stream_drop(h->stream); /* drop this fragment. */
         } else {
             /* store this fragment's data, start feeding it to SDL. */
             /*printf("PULSEAUDIO: captured %d new bytes\n", (int) nbytes);*/
-            h->capturebuf = (const Uint8 *) data;
+            h->capturebuf = (const Uint8 *)data;
             h->capturelen = nbytes;
         }
     }
 
-    return -1;  /* not enabled? */
+    return -1; /* not enabled? */
 }
 
-static void
-PULSEAUDIO_FlushCapture(_THIS)
+static void PULSEAUDIO_FlushCapture(_THIS)
 {
     struct SDL_PrivateAudioData *h = this->hidden;
     const void *data = NULL;
@@ -473,21 +453,20 @@ PULSEAUDIO_FlushCapture(_THIS)
             PULSEAUDIO_pa_stream_get_state(h->stream) != PA_STREAM_READY ||
             PULSEAUDIO_pa_mainloop_iterate(h->mainloop, 1, NULL) < 0) {
             SDL_OpenedAudioDeviceDisconnected(this);
-            return;  /* uhoh, pulse failed! */
+            return; /* uhoh, pulse failed! */
         }
 
         if (PULSEAUDIO_pa_stream_readable_size(h->stream) == 0) {
-            break;  /* no data available, so we're done. */
+            break; /* no data available, so we're done. */
         }
 
         /* a new fragment is available! Just dump it. */
         PULSEAUDIO_pa_stream_peek(h->stream, &data, &nbytes);
-        PULSEAUDIO_pa_stream_drop(h->stream);  /* drop this fragment. */
+        PULSEAUDIO_pa_stream_drop(h->stream); /* drop this fragment. */
     }
 }
 
-static void
-PULSEAUDIO_CloseDevice(_THIS)
+static void PULSEAUDIO_CloseDevice(_THIS)
 {
     if (this->hidden->stream) {
         if (this->hidden->capturebuf != NULL) {
@@ -503,48 +482,44 @@ PULSEAUDIO_CloseDevice(_THIS)
     SDL_free(this->hidden);
 }
 
-static void
-SinkDeviceNameCallback(pa_context *c, const pa_sink_info *i, int is_last, void *data)
+static void SinkDeviceNameCallback(pa_context *c, const pa_sink_info *i, int is_last, void *data)
 {
     if (i) {
-        char **devname = (char **) data;
+        char **devname = (char **)data;
         *devname = SDL_strdup(i->name);
     }
 }
 
-static void
-SourceDeviceNameCallback(pa_context *c, const pa_source_info *i, int is_last, void *data)
+static void SourceDeviceNameCallback(pa_context *c, const pa_source_info *i, int is_last, void *data)
 {
     if (i) {
-        char **devname = (char **) data;
+        char **devname = (char **)data;
         *devname = SDL_strdup(i->name);
     }
 }
 
-static SDL_bool
-FindDeviceName(struct SDL_PrivateAudioData *h, const SDL_bool iscapture, void *handle)
+static SDL_bool FindDeviceName(struct SDL_PrivateAudioData *h, const SDL_bool iscapture, void *handle)
 {
-    const uint32_t idx = ((uint32_t) ((size_t) handle)) - 1;
+    const uint32_t idx = ((uint32_t)((intptr_t)handle)) - 1;
 
-    if (handle == NULL) {  /* NULL == default device. */
+    if (handle == NULL) { /* NULL == default device. */
         return SDL_TRUE;
     }
 
     if (iscapture) {
         WaitForPulseOperation(h->mainloop,
-            PULSEAUDIO_pa_context_get_source_info_by_index(h->context, idx,
-                SourceDeviceNameCallback, &h->device_name));
+                              PULSEAUDIO_pa_context_get_source_info_by_index(h->context, idx,
+                                                                             SourceDeviceNameCallback, &h->device_name));
     } else {
         WaitForPulseOperation(h->mainloop,
-            PULSEAUDIO_pa_context_get_sink_info_by_index(h->context, idx,
-                SinkDeviceNameCallback, &h->device_name));
+                              PULSEAUDIO_pa_context_get_sink_info_by_index(h->context, idx,
+                                                                           SinkDeviceNameCallback, &h->device_name));
     }
 
-    return (h->device_name != NULL);
+    return h->device_name != NULL;
 }
 
-static int
-PULSEAUDIO_OpenDevice(_THIS, const char *devname)
+static int PULSEAUDIO_OpenDevice(_THIS, const char *devname)
 {
     struct SDL_PrivateAudioData *h = NULL;
     SDL_AudioFormat test_format;
@@ -558,8 +533,7 @@ PULSEAUDIO_OpenDevice(_THIS, const char *devname)
     int rc = 0;
 
     /* Initialize all variables that we clean on shutdown */
-    h = this->hidden = (struct SDL_PrivateAudioData *)
-        SDL_malloc((sizeof *this->hidden));
+    h = this->hidden = (struct SDL_PrivateAudioData *)SDL_malloc(sizeof(*this->hidden));
     if (this->hidden == NULL) {
         return SDL_OutOfMemory();
     }
@@ -604,15 +578,12 @@ PULSEAUDIO_OpenDevice(_THIS, const char *devname)
     paspec.format = format;
 
     /* Calculate the final parameters for this audio specification */
-#ifdef PA_STREAM_ADJUST_LATENCY
-    this->spec.samples /= 2; /* Mix in smaller chunck to avoid underruns */
-#endif
     SDL_CalculateAudioSpec(&this->spec);
 
     /* Allocate mixing buffer */
     if (!iscapture) {
         h->mixlen = this->spec.size;
-        h->mixbuf = (Uint8 *) SDL_malloc(h->mixlen);
+        h->mixbuf = (Uint8 *)SDL_malloc(h->mixlen);
         if (h->mixbuf == NULL) {
             return SDL_OutOfMemory();
         }
@@ -623,22 +594,12 @@ PULSEAUDIO_OpenDevice(_THIS, const char *devname)
     paspec.rate = this->spec.freq;
 
     /* Reduced prebuffering compared to the defaults. */
-#ifdef PA_STREAM_ADJUST_LATENCY
     paattr.fragsize = this->spec.size;
-    /* 2x original requested bufsize */
-    paattr.tlength = h->mixlen * 4;
+    paattr.tlength = h->mixlen;
     paattr.prebuf = -1;
     paattr.maxlength = -1;
-    /* -1 can lead to pa_stream_writable_size() >= mixlen never being true */
-    paattr.minreq = h->mixlen;
-    flags = PA_STREAM_ADJUST_LATENCY;
-#else
-    paattr.fragsize = this->spec.size;
-    paattr.tlength = h->mixlen*2;
-    paattr.prebuf = h->mixlen*2;
-    paattr.maxlength = h->mixlen*2;
-    paattr.minreq = h->mixlen;
-#endif
+    paattr.minreq = -1;
+    flags |= PA_STREAM_ADJUST_LATENCY;
 
     if (ConnectToPulseServer(&h->mainloop, &h->context) < 0) {
         return SDL_SetError("Could not connect to PulseAudio server");
@@ -658,9 +619,9 @@ PULSEAUDIO_OpenDevice(_THIS, const char *devname)
     h->stream = PULSEAUDIO_pa_stream_new(
         h->context,
         (name && *name) ? name : "Audio Stream", /* stream description */
-        &paspec,    /* sample format spec */
-        &pacmap     /* channel map */
-        );
+        &paspec,                                 /* sample format spec */
+        &pacmap                                  /* channel map */
+    );
 
     if (h->stream == NULL) {
         return SDL_SetError("Could not set up PulseAudio stream");
@@ -675,6 +636,7 @@ PULSEAUDIO_OpenDevice(_THIS, const char *devname)
     if (iscapture) {
         rc = PULSEAUDIO_pa_stream_connect_record(h->stream, h->device_name, &paattr, flags);
     } else {
+        PULSEAUDIO_pa_stream_set_write_callback(h->stream, WriteCallback, h);
         rc = PULSEAUDIO_pa_stream_connect_playback(h->stream, h->device_name, &paattr, flags, NULL, NULL);
     }
 
@@ -700,10 +662,16 @@ static pa_mainloop *hotplug_mainloop = NULL;
 static pa_context *hotplug_context = NULL;
 static SDL_Thread *hotplug_thread = NULL;
 
+/* These are the OS identifiers (i.e. ALSA strings)... */
+static char *default_sink_path = NULL;
+static char *default_source_path = NULL;
+/* ... and these are the descriptions we use in GetDefaultAudioInfo. */
+static char *default_sink_name = NULL;
+static char *default_source_name = NULL;
+
 /* device handles are device index + 1, cast to void*, so we never pass a NULL. */
 
-static SDL_AudioFormat
-PulseFormatToSDLFormat(pa_sample_format_t format)
+static SDL_AudioFormat PulseFormatToSDLFormat(pa_sample_format_t format)
 {
     switch (format) {
     case PA_SAMPLE_U8:
@@ -726,10 +694,10 @@ PulseFormatToSDLFormat(pa_sample_format_t format)
 }
 
 /* This is called when PulseAudio adds an output ("sink") device. */
-static void
-SinkInfoCallback(pa_context *c, const pa_sink_info *i, int is_last, void *data)
+static void SinkInfoCallback(pa_context *c, const pa_sink_info *i, int is_last, void *data)
 {
     SDL_AudioSpec spec;
+    SDL_bool add = (SDL_bool)((intptr_t)data);
     if (i) {
         spec.freq = i->sample_spec.rate;
         spec.channels = i->sample_spec.channels;
@@ -740,15 +708,24 @@ SinkInfoCallback(pa_context *c, const pa_sink_info *i, int is_last, void *data)
         spec.callback = NULL;
         spec.userdata = NULL;
 
-        SDL_AddAudioDevice(SDL_FALSE, i->description, &spec, (void *) ((size_t) i->index+1));
+        if (add) {
+            SDL_AddAudioDevice(SDL_FALSE, i->description, &spec, (void *)((intptr_t)i->index + 1));
+        }
+
+        if (default_sink_path != NULL && SDL_strcmp(i->name, default_sink_path) == 0) {
+            if (default_sink_name != NULL) {
+                SDL_free(default_sink_name);
+            }
+            default_sink_name = SDL_strdup(i->description);
+        }
     }
 }
 
 /* This is called when PulseAudio adds a capture ("source") device. */
-static void
-SourceInfoCallback(pa_context *c, const pa_source_info *i, int is_last, void *data)
+static void SourceInfoCallback(pa_context *c, const pa_source_info *i, int is_last, void *data)
 {
     SDL_AudioSpec spec;
+    SDL_bool add = (SDL_bool)((intptr_t)data);
     if (i) {
         /* Maybe skip "monitor" sources. These are just output from other sinks. */
         if (include_monitors || (i->monitor_of_sink == PA_INVALID_INDEX)) {
@@ -761,59 +738,115 @@ SourceInfoCallback(pa_context *c, const pa_source_info *i, int is_last, void *da
             spec.callback = NULL;
             spec.userdata = NULL;
 
-            SDL_AddAudioDevice(SDL_TRUE, i->description, &spec, (void *) ((size_t) i->index+1));
+            if (add) {
+                SDL_AddAudioDevice(SDL_TRUE, i->description, &spec, (void *)((intptr_t)i->index + 1));
+            }
+
+            if (default_source_path != NULL && SDL_strcmp(i->name, default_source_path) == 0) {
+                if (default_source_name != NULL) {
+                    SDL_free(default_source_name);
+                }
+                default_source_name = SDL_strdup(i->description);
+            }
         }
     }
 }
 
+static void ServerInfoCallback(pa_context *c, const pa_server_info *i, void *data)
+{
+    if (default_sink_path != NULL) {
+        SDL_free(default_sink_path);
+    }
+    if (default_source_path != NULL) {
+        SDL_free(default_source_path);
+    }
+    default_sink_path = SDL_strdup(i->default_sink_name);
+    default_source_path = SDL_strdup(i->default_source_name);
+}
+
 /* This is called when PulseAudio has a device connected/removed/changed. */
-static void
-HotplugCallback(pa_context *c, pa_subscription_event_type_t t, uint32_t idx, void *data)
+static void HotplugCallback(pa_context *c, pa_subscription_event_type_t t, uint32_t idx, void *data)
 {
     const SDL_bool added = ((t & PA_SUBSCRIPTION_EVENT_TYPE_MASK) == PA_SUBSCRIPTION_EVENT_NEW);
     const SDL_bool removed = ((t & PA_SUBSCRIPTION_EVENT_TYPE_MASK) == PA_SUBSCRIPTION_EVENT_REMOVE);
+    const SDL_bool changed = ((t & PA_SUBSCRIPTION_EVENT_TYPE_MASK) == PA_SUBSCRIPTION_EVENT_CHANGE);
 
-    if (added || removed) {  /* we only care about add/remove events. */
+    if (added || removed || changed) { /* we only care about add/remove events. */
         const SDL_bool sink = ((t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) == PA_SUBSCRIPTION_EVENT_SINK);
         const SDL_bool source = ((t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) == PA_SUBSCRIPTION_EVENT_SOURCE);
 
         /* adds need sink details from the PulseAudio server. Another callback... */
-        if (added && sink) {
-            PULSEAUDIO_pa_context_get_sink_info_by_index(hotplug_context, idx, SinkInfoCallback, NULL);
-        } else if (added && source) {
-            PULSEAUDIO_pa_context_get_source_info_by_index(hotplug_context, idx, SourceInfoCallback, NULL);
+        if ((added || changed) && sink) {
+            if (changed) {
+                PULSEAUDIO_pa_context_get_server_info(hotplug_context, ServerInfoCallback, NULL);
+            }
+            PULSEAUDIO_pa_context_get_sink_info_by_index(hotplug_context, idx, SinkInfoCallback, (void *)((intptr_t)added));
+        } else if ((added || changed) && source) {
+            if (changed) {
+                PULSEAUDIO_pa_context_get_server_info(hotplug_context, ServerInfoCallback, NULL);
+            }
+            PULSEAUDIO_pa_context_get_source_info_by_index(hotplug_context, idx, SourceInfoCallback, (void *)((intptr_t)added));
         } else if (removed && (sink || source)) {
             /* removes we can handle just with the device index. */
-            SDL_RemoveAudioDevice(source != 0, (void *) ((size_t) idx+1));
+            SDL_RemoveAudioDevice(source != 0, (void *)((intptr_t)idx + 1));
         }
     }
 }
 
 /* this runs as a thread while the Pulse target is initialized to catch hotplug events. */
-static int SDLCALL
-HotplugThread(void *data)
+static int SDLCALL HotplugThread(void *data)
 {
     pa_operation *o;
     SDL_SetThreadPriority(SDL_THREAD_PRIORITY_LOW);
     PULSEAUDIO_pa_context_set_subscribe_callback(hotplug_context, HotplugCallback, NULL);
     o = PULSEAUDIO_pa_context_subscribe(hotplug_context, PA_SUBSCRIPTION_MASK_SINK | PA_SUBSCRIPTION_MASK_SOURCE, NULL, NULL);
-    PULSEAUDIO_pa_operation_unref(o);  /* don't wait for it, just do our thing. */
+    PULSEAUDIO_pa_operation_unref(o); /* don't wait for it, just do our thing. */
     PULSEAUDIO_pa_mainloop_run(hotplug_mainloop, NULL);
     return 0;
 }
 
-static void
-PULSEAUDIO_DetectDevices()
+static void PULSEAUDIO_DetectDevices()
 {
-    WaitForPulseOperation(hotplug_mainloop, PULSEAUDIO_pa_context_get_sink_info_list(hotplug_context, SinkInfoCallback, NULL));
-    WaitForPulseOperation(hotplug_mainloop, PULSEAUDIO_pa_context_get_source_info_list(hotplug_context, SourceInfoCallback, NULL));
+    WaitForPulseOperation(hotplug_mainloop, PULSEAUDIO_pa_context_get_server_info(hotplug_context, ServerInfoCallback, NULL));
+    WaitForPulseOperation(hotplug_mainloop, PULSEAUDIO_pa_context_get_sink_info_list(hotplug_context, SinkInfoCallback, (void *)((intptr_t)SDL_TRUE)));
+    WaitForPulseOperation(hotplug_mainloop, PULSEAUDIO_pa_context_get_source_info_list(hotplug_context, SourceInfoCallback, (void *)((intptr_t)SDL_TRUE)));
 
     /* ok, we have a sane list, let's set up hotplug notifications now... */
     hotplug_thread = SDL_CreateThreadInternal(HotplugThread, "PulseHotplug", 256 * 1024, NULL);
 }
 
-static void
-PULSEAUDIO_Deinitialize(void)
+static int PULSEAUDIO_GetDefaultAudioInfo(char **name, SDL_AudioSpec *spec, int iscapture)
+{
+    int i;
+    int numdevices;
+
+    char *target;
+    if (iscapture) {
+        if (default_source_name == NULL) {
+            return SDL_SetError("PulseAudio could not find a default source");
+        }
+        target = default_source_name;
+    } else {
+        if (default_sink_name == NULL) {
+            return SDL_SetError("PulseAudio could not find a default sink");
+        }
+        target = default_sink_name;
+    }
+
+    numdevices = SDL_GetNumAudioDevices(iscapture);
+    for (i = 0; i < numdevices; i += 1) {
+        if (SDL_strcmp(SDL_GetAudioDeviceName(i, iscapture), target) == 0) {
+            if (name != NULL) {
+                *name = SDL_strdup(target);
+            }
+            SDL_GetAudioDeviceSpec(i, iscapture, spec);
+            return 0;
+        }
+    }
+    return SDL_SetError("Could not find default PulseAudio device");
+}
+
+static void PULSEAUDIO_Deinitialize(void)
 {
     if (hotplug_thread) {
         PULSEAUDIO_pa_mainloop_quit(hotplug_mainloop, 0);
@@ -825,11 +858,27 @@ PULSEAUDIO_Deinitialize(void)
     hotplug_mainloop = NULL;
     hotplug_context = NULL;
 
+    if (default_sink_path != NULL) {
+        SDL_free(default_sink_path);
+        default_sink_path = NULL;
+    }
+    if (default_source_path != NULL) {
+        SDL_free(default_source_path);
+        default_source_path = NULL;
+    }
+    if (default_sink_name != NULL) {
+        SDL_free(default_sink_name);
+        default_sink_name = NULL;
+    }
+    if (default_source_name != NULL) {
+        SDL_free(default_source_name);
+        default_source_name = NULL;
+    }
+
     UnloadPulseAudioLibrary();
 }
 
-static SDL_bool
-PULSEAUDIO_Init(SDL_AudioDriverImpl * impl)
+static SDL_bool PULSEAUDIO_Init(SDL_AudioDriverImpl *impl)
 {
     if (LoadPulseAudioLibrary() < 0) {
         return SDL_FALSE;
@@ -852,10 +901,12 @@ PULSEAUDIO_Init(SDL_AudioDriverImpl * impl)
     impl->Deinitialize = PULSEAUDIO_Deinitialize;
     impl->CaptureFromDevice = PULSEAUDIO_CaptureFromDevice;
     impl->FlushCapture = PULSEAUDIO_FlushCapture;
+    impl->GetDefaultAudioInfo = PULSEAUDIO_GetDefaultAudioInfo;
 
     impl->HasCaptureSupport = SDL_TRUE;
+    impl->SupportsNonPow2Samples = SDL_TRUE;
 
-    return SDL_TRUE;   /* this audio target is available. */
+    return SDL_TRUE; /* this audio target is available. */
 }
 
 AudioBootStrap PULSEAUDIO_bootstrap = {
