@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2021 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -28,6 +28,11 @@
 #include "../../core/windows/SDL_directx.h"
 #include "../../core/windows/SDL_xinput.h"
 
+/* Set up for C function definitions, even when using C++ */
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
  * Haptic system hardware data.
  */
@@ -36,17 +41,16 @@ struct haptic_hwdata
 #if SDL_HAPTIC_DINPUT
     LPDIRECTINPUTDEVICE8 device;
 #endif
-    DWORD axes[3];              /* Axes to use. */
-    SDL_bool is_joystick;       /* Device is loaded as joystick. */
-    Uint8 bXInputHaptic; /* Supports force feedback via XInput. */
-    Uint8 userid; /* XInput userid index for this joystick */
+    DWORD axes[3];        /* Axes to use. */
+    SDL_bool is_joystick; /* Device is loaded as joystick. */
+    Uint8 bXInputHaptic;  /* Supports force feedback via XInput. */
+    Uint8 userid;         /* XInput userid index for this joystick */
     SDL_Thread *thread;
     SDL_mutex *mutex;
     Uint32 stopTicks;
     SDL_atomic_t stopThread;
 };
 
-
 /*
  * Haptic system effect data.
  */
@@ -64,8 +68,8 @@ struct haptic_hweffect
 #endif
 
 /*
-* List of available haptic devices.
-*/
+ * List of available haptic devices.
+ */
 typedef struct SDL_hapticlist_item
 {
     char *name;
@@ -75,7 +79,7 @@ typedef struct SDL_hapticlist_item
     DIDEVCAPS capabilities;
 #endif
     SDL_bool bXInputHaptic; /* Supports force feedback via XInput. */
-    Uint8 userid; /* XInput userid index for this joystick */
+    Uint8 userid;           /* XInput userid index for this joystick */
     struct SDL_hapticlist_item *next;
 } SDL_hapticlist_item;
 
@@ -84,7 +88,11 @@ extern SDL_hapticlist_item *SDL_hapticlist;
 extern int SDL_SYS_AddHapticDevice(SDL_hapticlist_item *item);
 extern int SDL_SYS_RemoveHapticDevice(SDL_hapticlist_item *prev, SDL_hapticlist_item *item);
 
+/* Ends C function definitions when using C++ */
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* SDL_windowshaptic_c_h_ */
 
 /* vi: set ts=4 sw=4 expandtab: */
-
