@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -53,12 +53,11 @@ typedef struct SDL_cond_generic
 } SDL_cond_generic;
 
 /* Create a condition variable */
-SDL_cond *
-SDL_CreateCond_generic(void)
+SDL_cond *SDL_CreateCond_generic(void)
 {
     SDL_cond_generic *cond;
 
-    cond = (SDL_cond_generic *) SDL_malloc(sizeof(SDL_cond_generic));
+    cond = (SDL_cond_generic *)SDL_malloc(sizeof(SDL_cond_generic));
     if (cond) {
         cond->lock = SDL_CreateMutex();
         cond->wait_sem = SDL_CreateSemaphore(0);
@@ -75,8 +74,7 @@ SDL_CreateCond_generic(void)
 }
 
 /* Destroy a condition variable */
-void
-SDL_DestroyCond_generic(SDL_cond * _cond)
+void SDL_DestroyCond_generic(SDL_cond *_cond)
 {
     SDL_cond_generic *cond = (SDL_cond_generic *)_cond;
     if (cond) {
@@ -94,12 +92,11 @@ SDL_DestroyCond_generic(SDL_cond * _cond)
 }
 
 /* Restart one of the threads that are waiting on the condition variable */
-int
-SDL_CondSignal_generic(SDL_cond * _cond)
+int SDL_CondSignal_generic(SDL_cond *_cond)
 {
     SDL_cond_generic *cond = (SDL_cond_generic *)_cond;
-    if (!cond) {
-        return SDL_SetError("Passed a NULL condition variable");
+    if (cond == NULL) {
+        return SDL_InvalidParamError("cond");
     }
 
     /* If there are waiting threads not already signalled, then
@@ -119,12 +116,11 @@ SDL_CondSignal_generic(SDL_cond * _cond)
 }
 
 /* Restart all threads that are waiting on the condition variable */
-int
-SDL_CondBroadcast_generic(SDL_cond * _cond)
+int SDL_CondBroadcast_generic(SDL_cond *_cond)
 {
     SDL_cond_generic *cond = (SDL_cond_generic *)_cond;
-    if (!cond) {
-        return SDL_SetError("Passed a NULL condition variable");
+    if (cond == NULL) {
+        return SDL_InvalidParamError("cond");
     }
 
     /* If there are waiting threads not already signalled, then
@@ -174,14 +170,13 @@ Thread B:
     SDL_CondSignal(cond);
     SDL_UnlockMutex(lock);
  */
-int
-SDL_CondWaitTimeout_generic(SDL_cond * _cond, SDL_mutex * mutex, Uint32 ms)
+int SDL_CondWaitTimeout_generic(SDL_cond *_cond, SDL_mutex *mutex, Uint32 ms)
 {
     SDL_cond_generic *cond = (SDL_cond_generic *)_cond;
     int retval;
 
-    if (!cond) {
-        return SDL_SetError("Passed a NULL condition variable");
+    if (cond == NULL) {
+        return SDL_InvalidParamError("cond");
     }
 
     /* Obtain the protection mutex, and increment the number of waiters.
@@ -230,8 +225,7 @@ SDL_CondWaitTimeout_generic(SDL_cond * _cond, SDL_mutex * mutex, Uint32 ms)
 }
 
 /* Wait on the condition variable forever */
-int
-SDL_CondWait_generic(SDL_cond * cond, SDL_mutex * mutex)
+int SDL_CondWait_generic(SDL_cond *cond, SDL_mutex *mutex)
 {
     return SDL_CondWaitTimeout_generic(cond, mutex, SDL_MUTEX_MAXWAIT);
 }
