@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -305,17 +305,18 @@ static Uint8 SDL_NACL_translate_mouse_button(int32_t button) {
     }
 }
 
-static SDL_Scancode
-SDL_NACL_translate_keycode(int keycode)
+static SDL_Scancode SDL_NACL_translate_keycode(int keycode)
 {
     SDL_Scancode scancode = SDL_SCANCODE_UNKNOWN;
 
     if (keycode < SDL_arraysize(NACL_Keycodes)) {
         scancode = NACL_Keycodes[keycode];
     }
+#ifdef DEBUG_SCANCODES
     if (scancode == SDL_SCANCODE_UNKNOWN) {
         SDL_Log("The key you just pressed is not recognized by SDL. To help get this fixed, please report this to the SDL forums/mailing list <https://discourse.libsdl.org/> NACL KeyCode %d", keycode);
     }
+#endif
     return scancode;
 }
 
