@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2021 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -26,7 +26,6 @@
 
 #include "SDL_waylanddyn.h"
 
-
 #ifdef SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC
 
 #include "SDL_name.h"
@@ -52,23 +51,23 @@ typedef struct
 #endif
 
 static waylanddynlib waylandlibs[] = {
-    {NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC},
-    {NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_EGL},
-    {NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_CURSOR},
-    {NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_XKBCOMMON},
-    {NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_LIBDECOR}
+    { NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC },
+    { NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_EGL },
+    { NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_CURSOR },
+    { NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_XKBCOMMON },
+    { NULL, SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_LIBDECOR }
 };
 
-static void *
-WAYLAND_GetSym(const char *fnname, int *pHasModule)
+static void *WAYLAND_GetSym(const char *fnname, int *pHasModule, SDL_bool required)
 {
     int i;
     void *fn = NULL;
     for (i = 0; i < SDL_TABLESIZE(waylandlibs); i++) {
         if (waylandlibs[i].lib != NULL) {
             fn = SDL_LoadFunction(waylandlibs[i].lib, fnname);
-            if (fn != NULL)
+            if (fn != NULL) {
                 break;
+            }
         }
     }
 
@@ -79,38 +78,43 @@ WAYLAND_GetSym(const char *fnname, int *pHasModule)
         SDL_Log("WAYLAND: Symbol '%s' NOT FOUND!\n", fnname);
 #endif
 
-    if (fn == NULL)
-        *pHasModule = 0;  /* kill this module. */
+    if (fn == NULL && required) {
+        *pHasModule = 0; /* kill this module. */
+    }
 
     return fn;
 }
 
+#else
+
+#include <wayland-egl.h>
+
 #endif /* SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC */
 
 /* Define all the function pointers and wrappers... */
-#define SDL_WAYLAND_MODULE(modname) int SDL_WAYLAND_HAVE_##modname = 0;
-#define SDL_WAYLAND_SYM(rc,fn,params) SDL_DYNWAYLANDFN_##fn WAYLAND_##fn = NULL;
-#define SDL_WAYLAND_INTERFACE(iface) const struct wl_interface *WAYLAND_##iface = NULL;
+#define SDL_WAYLAND_MODULE(modname)         int SDL_WAYLAND_HAVE_##modname = 0;
+#define SDL_WAYLAND_SYM(rc, fn, params)     SDL_DYNWAYLANDFN_##fn WAYLAND_##fn = NULL;
+#define SDL_WAYLAND_SYM_OPT(rc, fn, params) SDL_DYNWAYLANDFN_##fn WAYLAND_##fn = NULL;
+#define SDL_WAYLAND_INTERFACE(iface)        const struct wl_interface *WAYLAND_##iface = NULL;
 #include "SDL_waylandsym.h"
 
 static int wayland_load_refcount = 0;
 
-void
-SDL_WAYLAND_UnloadSymbols(void)
+void SDL_WAYLAND_UnloadSymbols(void)
 {
     /* Don't actually unload if more than one module is using the libs... */
     if (wayland_load_refcount > 0) {
         if (--wayland_load_refcount == 0) {
-#ifdef SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC            
+#ifdef SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC
             int i;
 #endif
-            
-            /* set all the function pointers to NULL. */
-#define SDL_WAYLAND_MODULE(modname) SDL_WAYLAND_HAVE_##modname = 0;
-#define SDL_WAYLAND_SYM(rc,fn,params) WAYLAND_##fn = NULL;
-#define SDL_WAYLAND_INTERFACE(iface) WAYLAND_##iface = NULL;
-#include "SDL_waylandsym.h"
 
+            /* set all the function pointers to NULL. */
+#define SDL_WAYLAND_MODULE(modname)         SDL_WAYLAND_HAVE_##modname = 0;
+#define SDL_WAYLAND_SYM(rc, fn, params)     WAYLAND_##fn = NULL;
+#define SDL_WAYLAND_SYM_OPT(rc, fn, params) WAYLAND_##fn = NULL;
+#define SDL_WAYLAND_INTERFACE(iface)        WAYLAND_##iface = NULL;
+#include "SDL_waylandsym.h"
 
 #ifdef SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC
             for (i = 0; i < SDL_TABLESIZE(waylandlibs); i++) {
@@ -125,10 +129,9 @@ SDL_WAYLAND_UnloadSymbols(void)
 }
 
 /* returns non-zero if all needed symbols were loaded. */
-int
-SDL_WAYLAND_LoadSymbols(void)
+int SDL_WAYLAND_LoadSymbols(void)
 {
-    int rc = 1;                 /* always succeed if not using Dynamic WAYLAND stuff. */
+    int rc = 1; /* always succeed if not using Dynamic WAYLAND stuff. */
 
     /* deal with multiple modules (dga, wayland, etc) needing these symbols... */
     if (wayland_load_refcount++ == 0) {
@@ -144,9 +147,10 @@ SDL_WAYLAND_LoadSymbols(void)
 #define SDL_WAYLAND_MODULE(modname) SDL_WAYLAND_HAVE_##modname = 1; /* default yes */
 #include "SDL_waylandsym.h"
 
-#define SDL_WAYLAND_MODULE(modname) thismod = &SDL_WAYLAND_HAVE_##modname;
-#define SDL_WAYLAND_SYM(rc,fn,params) WAYLAND_##fn = (SDL_DYNWAYLANDFN_##fn) WAYLAND_GetSym(#fn,thismod);
-#define SDL_WAYLAND_INTERFACE(iface) WAYLAND_##iface = (struct wl_interface *) WAYLAND_GetSym(#iface,thismod);
+#define SDL_WAYLAND_MODULE(modname)         thismod = &SDL_WAYLAND_HAVE_##modname;
+#define SDL_WAYLAND_SYM(rc, fn, params)     WAYLAND_##fn = (SDL_DYNWAYLANDFN_##fn)WAYLAND_GetSym(#fn, thismod, SDL_TRUE);
+#define SDL_WAYLAND_SYM_OPT(rc, fn, params) WAYLAND_##fn = (SDL_DYNWAYLANDFN_##fn)WAYLAND_GetSym(#fn, thismod, SDL_FALSE);
+#define SDL_WAYLAND_INTERFACE(iface)        WAYLAND_##iface = (struct wl_interface *)WAYLAND_GetSym(#iface, thismod, SDL_TRUE);
 #include "SDL_waylandsym.h"
 
         if (SDL_WAYLAND_HAVE_WAYLAND_CLIENT) {
@@ -158,11 +162,12 @@ SDL_WAYLAND_LoadSymbols(void)
             rc = 0;
         }
 
-#else  /* no dynamic WAYLAND */
+#else /* no dynamic WAYLAND */
 
-#define SDL_WAYLAND_MODULE(modname) SDL_WAYLAND_HAVE_##modname = 1; /* default yes */
-#define SDL_WAYLAND_SYM(rc,fn,params) WAYLAND_##fn = fn;
-#define SDL_WAYLAND_INTERFACE(iface) WAYLAND_##iface = &iface;
+#define SDL_WAYLAND_MODULE(modname)         SDL_WAYLAND_HAVE_##modname = 1; /* default yes */
+#define SDL_WAYLAND_SYM(rc, fn, params)     WAYLAND_##fn = fn;
+#define SDL_WAYLAND_SYM_OPT(rc, fn, params) WAYLAND_##fn = fn;
+#define SDL_WAYLAND_INTERFACE(iface)        WAYLAND_##iface = &iface;
 #include "SDL_waylandsym.h"
 
 #endif
