@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -28,18 +28,15 @@
 #include "SDL_x11xinput2.h"
 #include "../../events/SDL_mouse_c.h"
 
-
 /* FIXME: Find a better place to put this... */
 static Cursor x11_empty_cursor = None;
 
-static Display *
-GetDisplay(void)
+static Display *GetDisplay(void)
 {
     return ((SDL_VideoData *)SDL_GetVideoDevice()->driverdata)->display;
 }
 
-static Cursor
-X11_CreateEmptyCursor()
+static Cursor X11_CreateEmptyCursor()
 {
     if (x11_empty_cursor == None) {
         Display *display = GetDisplay();
@@ -50,18 +47,17 @@ X11_CreateEmptyCursor()
         SDL_zeroa(data);
         color.red = color.green = color.blue = 0;
         pixmap = X11_XCreateBitmapFromData(display, DefaultRootWindow(display),
-                                       data, 1, 1);
+                                           data, 1, 1);
         if (pixmap) {
             x11_empty_cursor = X11_XCreatePixmapCursor(display, pixmap, pixmap,
-                                                   &color, &color, 0, 0);
+                                                       &color, &color, 0, 0);
             X11_XFreePixmap(display, pixmap);
         }
     }
     return x11_empty_cursor;
 }
 
-static void
-X11_DestroyEmptyCursor(void)
+static void X11_DestroyEmptyCursor(void)
 {
     if (x11_empty_cursor != None) {
         X11_XFreeCursor(GetDisplay(), x11_empty_cursor);
@@ -69,15 +65,14 @@ X11_DestroyEmptyCursor(void)
     }
 }
 
-static SDL_Cursor *
-X11_CreateDefaultCursor()
+static SDL_Cursor *X11_CreateDefaultCursor()
 {
     SDL_Cursor *cursor;
 
     cursor = SDL_calloc(1, sizeof(*cursor));
     if (cursor) {
         /* None is used to indicate the default cursor */
-        cursor->driverdata = (void*)(uintptr_t)None;
+        cursor->driverdata = (void *)(uintptr_t)None;
     } else {
         SDL_OutOfMemory();
     }
@@ -86,15 +81,14 @@ X11_CreateDefaultCursor()
 }
 
 #if SDL_VIDEO_DRIVER_X11_XCURSOR
-static Cursor
-X11_CreateXCursorCursor(SDL_Surface * surface, int hot_x, int hot_y)
+static Cursor X11_CreateXCursorCursor(SDL_Surface *surface, int hot_x, int hot_y)
 {
     Display *display = GetDisplay();
     Cursor cursor = None;
     XcursorImage *image;
 
     image = X11_XcursorImageCreate(surface->w, surface->h);
-    if (!image) {
+    if (image == NULL) {
         SDL_OutOfMemory();
         return None;
     }
@@ -104,7 +98,7 @@ X11_CreateXCursorCursor(SDL_Surface * surface, int hot_x, int hot_y)
 
     SDL_assert(surface->format->format == SDL_PIXELFORMAT_ARGB8888);
     SDL_assert(surface->pitch == surface->w * 4);
-    SDL_memcpy(image->pixels, surface->pixels, surface->h * surface->pitch);
+    SDL_memcpy(image->pixels, surface->pixels, (size_t)surface->h * surface->pitch);
 
     cursor = X11_XcursorImageLoadCursor(display, image);
 
@@ -114,8 +108,7 @@ X11_CreateXCursorCursor(SDL_Surface * surface, int hot_x, int hot_y)
 }
 #endif /* SDL_VIDEO_DRIVER_X11_XCURSOR */
 
-static Cursor
-X11_CreatePixmapCursor(SDL_Surface * surface, int hot_x, int hot_y)
+static Cursor X11_CreatePixmapCursor(SDL_Surface *surface, int hot_x, int hot_y)
 {
     Display *display = GetDisplay();
     XColor fg, bg;
@@ -125,16 +118,16 @@ X11_CreatePixmapCursor(SDL_Surface * surface, int hot_x, int hot_y)
     Pixmap data_pixmap, mask_pixmap;
     int x, y;
     unsigned int rfg, gfg, bfg, rbg, gbg, bbg, fgBits, bgBits;
-    unsigned int width_bytes = ((surface->w + 7) & ~7) / 8;
+    size_t width_bytes = ((surface->w + 7) & ~((size_t)7)) / 8;
 
     data_bits = SDL_calloc(1, surface->h * width_bytes);
-    if (!data_bits) {
+    if (data_bits == NULL) {
         SDL_OutOfMemory();
         return None;
     }
 
     mask_bits = SDL_calloc(1, surface->h * width_bytes);
-    if (!mask_bits) {
+    if (mask_bits == NULL) {
         SDL_free(data_bits);
         SDL_OutOfMemory();
         return None;
@@ -148,9 +141,9 @@ X11_CreatePixmapCursor(SDL_Surface * surface, int hot_x, int hot_y)
         ptr = (Uint32 *)((Uint8 *)surface->pixels + y * surface->pitch);
         for (x = 0; x < surface->w; ++x) {
             int alpha = (*ptr >> 24) & 0xff;
-            int red   = (*ptr >> 16) & 0xff;
+            int red = (*ptr >> 16) & 0xff;
             int green = (*ptr >> 8) & 0xff;
-            int blue  = (*ptr >> 0) & 0xff;
+            int blue = (*ptr >> 0) & 0xff;
             if (alpha > 25) {
                 mask_bits[y * width_bytes + x / 8] |= (0x01 << (x % 8));
 
@@ -172,27 +165,29 @@ X11_CreatePixmapCursor(SDL_Surface * surface, int hot_x, int hot_y)
     }
 
     if (fgBits) {
-        fg.red   = rfg * 257 / fgBits;
+        fg.red = rfg * 257 / fgBits;
         fg.green = gfg * 257 / fgBits;
-        fg.blue  = bfg * 257 / fgBits;
+        fg.blue = bfg * 257 / fgBits;
+    } else {
+        fg.red = fg.green = fg.blue = 0;
     }
-    else fg.red = fg.green = fg.blue = 0;
 
     if (bgBits) {
-        bg.red   = rbg * 257 / bgBits;
+        bg.red = rbg * 257 / bgBits;
         bg.green = gbg * 257 / bgBits;
-        bg.blue  = bbg * 257 / bgBits;
+        bg.blue = bbg * 257 / bgBits;
+    } else {
+        bg.red = bg.green = bg.blue = 0;
     }
-    else bg.red = bg.green = bg.blue = 0;
 
     data_pixmap = X11_XCreateBitmapFromData(display, DefaultRootWindow(display),
-                                        (char*)data_bits,
-                                        surface->w, surface->h);
+                                            (char *)data_bits,
+                                            surface->w, surface->h);
     mask_pixmap = X11_XCreateBitmapFromData(display, DefaultRootWindow(display),
-                                        (char*)mask_bits,
-                                        surface->w, surface->h);
+                                            (char *)mask_bits,
+                                            surface->w, surface->h);
     cursor = X11_XCreatePixmapCursor(display, data_pixmap, mask_pixmap,
-                                 &fg, &bg, hot_x, hot_y);
+                                     &fg, &bg, hot_x, hot_y);
     X11_XFreePixmap(display, data_pixmap);
     X11_XFreePixmap(display, mask_pixmap);
     SDL_free(data_bits);
@@ -201,8 +196,7 @@ X11_CreatePixmapCursor(SDL_Surface * surface, int hot_x, int hot_y)
     return cursor;
 }
 
-static SDL_Cursor *
-X11_CreateCursor(SDL_Surface * surface, int hot_x, int hot_y)
+static SDL_Cursor *X11_CreateCursor(SDL_Surface *surface, int hot_x, int hot_y)
 {
     SDL_Cursor *cursor;
 
@@ -218,7 +212,7 @@ X11_CreateCursor(SDL_Surface * surface, int hot_x, int hot_y)
         if (x11_cursor == None) {
             x11_cursor = X11_CreatePixmapCursor(surface, hot_x, hot_y);
         }
-        cursor->driverdata = (void*)(uintptr_t)x11_cursor;
+        cursor->driverdata = (void *)(uintptr_t)x11_cursor;
     } else {
         SDL_OutOfMemory();
     }
@@ -226,31 +220,53 @@ X11_CreateCursor(SDL_Surface * surface, int hot_x, int hot_y)
     return cursor;
 }
 
-static SDL_Cursor *
-X11_CreateSystemCursor(SDL_SystemCursor id)
+static SDL_Cursor *X11_CreateSystemCursor(SDL_SystemCursor id)
 {
     SDL_Cursor *cursor;
     unsigned int shape;
 
-    switch(id)
-    {
+    switch (id) {
     default:
         SDL_assert(0);
         return NULL;
     /* X Font Cursors reference: */
     /*   http://tronche.com/gui/x/xlib/appendix/b/ */
-    case SDL_SYSTEM_CURSOR_ARROW:     shape = XC_left_ptr; break;
-    case SDL_SYSTEM_CURSOR_IBEAM:     shape = XC_xterm; break;
-    case SDL_SYSTEM_CURSOR_WAIT:      shape = XC_watch; break;
-    case SDL_SYSTEM_CURSOR_CROSSHAIR: shape = XC_tcross; break;
-    case SDL_SYSTEM_CURSOR_WAITARROW: shape = XC_watch; break;
-    case SDL_SYSTEM_CURSOR_SIZENWSE:  shape = XC_fleur; break;
-    case SDL_SYSTEM_CURSOR_SIZENESW:  shape = XC_fleur; break;
-    case SDL_SYSTEM_CURSOR_SIZEWE:    shape = XC_sb_h_double_arrow; break;
-    case SDL_SYSTEM_CURSOR_SIZENS:    shape = XC_sb_v_double_arrow; break;
-    case SDL_SYSTEM_CURSOR_SIZEALL:   shape = XC_fleur; break;
-    case SDL_SYSTEM_CURSOR_NO:        shape = XC_pirate; break;
-    case SDL_SYSTEM_CURSOR_HAND:      shape = XC_hand2; break;
+    case SDL_SYSTEM_CURSOR_ARROW:
+        shape = XC_left_ptr;
+        break;
+    case SDL_SYSTEM_CURSOR_IBEAM:
+        shape = XC_xterm;
+        break;
+    case SDL_SYSTEM_CURSOR_WAIT:
+        shape = XC_watch;
+        break;
+    case SDL_SYSTEM_CURSOR_CROSSHAIR:
+        shape = XC_tcross;
+        break;
+    case SDL_SYSTEM_CURSOR_WAITARROW:
+        shape = XC_watch;
+        break;
+    case SDL_SYSTEM_CURSOR_SIZENWSE:
+        shape = XC_top_left_corner;
+        break;
+    case SDL_SYSTEM_CURSOR_SIZENESW:
+        shape = XC_top_right_corner;
+        break;
+    case SDL_SYSTEM_CURSOR_SIZEWE:
+        shape = XC_sb_h_double_arrow;
+        break;
+    case SDL_SYSTEM_CURSOR_SIZENS:
+        shape = XC_sb_v_double_arrow;
+        break;
+    case SDL_SYSTEM_CURSOR_SIZEALL:
+        shape = XC_fleur;
+        break;
+    case SDL_SYSTEM_CURSOR_NO:
+        shape = XC_pirate;
+        break;
+    case SDL_SYSTEM_CURSOR_HAND:
+        shape = XC_hand2;
+        break;
     }
 
     cursor = SDL_calloc(1, sizeof(*cursor));
@@ -259,7 +275,7 @@ X11_CreateSystemCursor(SDL_SystemCursor id)
 
         x11_cursor = X11_XCreateFontCursor(GetDisplay(), shape);
 
-        cursor->driverdata = (void*)(uintptr_t)x11_cursor;
+        cursor->driverdata = (void *)(uintptr_t)x11_cursor;
     } else {
         SDL_OutOfMemory();
     }
@@ -267,8 +283,7 @@ X11_CreateSystemCursor(SDL_SystemCursor id)
     return cursor;
 }
 
-static void
-X11_FreeCursor(SDL_Cursor * cursor)
+static void X11_FreeCursor(SDL_Cursor *cursor)
 {
     Cursor x11_cursor = (Cursor)cursor->driverdata;
 
@@ -278,8 +293,7 @@ X11_FreeCursor(SDL_Cursor * cursor)
     SDL_free(cursor);
 }
 
-static int
-X11_ShowCursor(SDL_Cursor * cursor)
+static int X11_ShowCursor(SDL_Cursor *cursor)
 {
     Cursor x11_cursor = 0;
 
@@ -310,20 +324,32 @@ X11_ShowCursor(SDL_Cursor * cursor)
     return 0;
 }
 
-static void
-WarpMouseInternal(Window xwindow, const int x, const int y)
+static void WarpMouseInternal(Window xwindow, const int x, const int y)
 {
-    SDL_VideoData *videodata = (SDL_VideoData *) SDL_GetVideoDevice()->driverdata;
+    SDL_VideoData *videodata = (SDL_VideoData *)SDL_GetVideoDevice()->driverdata;
     Display *display = videodata->display;
-    X11_XWarpPointer(display, None, xwindow, 0, 0, 0, 0, x, y);
+#if SDL_VIDEO_DRIVER_X11_XINPUT2
+    int deviceid = 0;
+    /* It seems XIWarpPointer() doesn't work correctly on multi-head setups:
+     * https://developer.blender.org/rB165caafb99c6846e53d11c4e966990aaffc06cea
+     */
+    if (ScreenCount(display) == 1) {
+        X11_XIGetClientPointer(display, None, &deviceid);
+    }
+    if (deviceid != 0) {
+        X11_XIWarpPointer(display, deviceid, None, xwindow, 0.0, 0.0, 0, 0, (double)x, (double)y);
+    } else
+#endif
+    {
+        X11_XWarpPointer(display, None, xwindow, 0, 0, 0, 0, x, y);
+    }
     X11_XSync(display, False);
     videodata->global_mouse_changed = SDL_TRUE;
 }
 
-static void
-X11_WarpMouse(SDL_Window * window, int x, int y)
+static void X11_WarpMouse(SDL_Window *window, int x, int y)
 {
-    SDL_WindowData *data = (SDL_WindowData *) window->driverdata;
+    SDL_WindowData *data = (SDL_WindowData *)window->driverdata;
 
 #if SDL_VIDEO_DRIVER_X11_XFIXES
     /* If we have no barrier, we need to warp */
@@ -335,39 +361,41 @@ X11_WarpMouse(SDL_Window * window, int x, int y)
 #endif
 }
 
-static int
-X11_WarpMouseGlobal(int x, int y)
+static int X11_WarpMouseGlobal(int x, int y)
 {
     WarpMouseInternal(DefaultRootWindow(GetDisplay()), x, y);
     return 0;
 }
 
-static int
-X11_SetRelativeMouseMode(SDL_bool enabled)
+static int X11_SetRelativeMouseMode(SDL_bool enabled)
 {
 #if SDL_VIDEO_DRIVER_X11_XINPUT2
-    if(X11_Xinput2IsInitialized())
+    if (X11_Xinput2IsInitialized()) {
         return 0;
+    }
 #else
     SDL_Unsupported();
 #endif
     return -1;
 }
 
-static int
-X11_CaptureMouse(SDL_Window *window)
+static int X11_CaptureMouse(SDL_Window *window)
 {
     Display *display = GetDisplay();
+    SDL_Window *mouse_focus = SDL_GetMouseFocus();
 
     if (window) {
-        SDL_WindowData *data = (SDL_WindowData *) window->driverdata;
+        SDL_WindowData *data = (SDL_WindowData *)window->driverdata;
         const unsigned int mask = ButtonPressMask | ButtonReleaseMask | PointerMotionMask | FocusChangeMask;
+        Window confined = (data->mouse_grabbed ? data->xwindow : None);
         const int rc = X11_XGrabPointer(display, data->xwindow, False,
                                         mask, GrabModeAsync, GrabModeAsync,
-                                        None, None, CurrentTime);
+                                        confined, None, CurrentTime);
         if (rc != GrabSuccess) {
             return SDL_SetError("X server refused mouse capture");
         }
+    } else if (mouse_focus) {
+        SDL_UpdateWindowGrab(mouse_focus);
     } else {
         X11_XUngrabPointer(display, CurrentTime);
     }
@@ -377,10 +405,9 @@ X11_CaptureMouse(SDL_Window *window)
     return 0;
 }
 
-static Uint32
-X11_GetGlobalMouseState(int *x, int *y)
+static Uint32 X11_GetGlobalMouseState(int *x, int *y)
 {
-    SDL_VideoData *videodata = (SDL_VideoData *) SDL_GetVideoDevice()->driverdata;
+    SDL_VideoData *videodata = (SDL_VideoData *)SDL_GetVideoDevice()->driverdata;
     Display *display = GetDisplay();
     const int num_screens = SDL_GetNumVideoDisplays();
     int i;
@@ -389,13 +416,16 @@ X11_GetGlobalMouseState(int *x, int *y)
 
 #if !SDL_VIDEO_DRIVER_X11_XINPUT2
     videodata->global_mouse_changed = SDL_TRUE;
+#else
+    if (!SDL_X11_HAVE_XINPUT2)
+        videodata->global_mouse_changed = SDL_TRUE;
 #endif
 
     /* check if we have this cached since XInput last saw the mouse move. */
     /* !!! FIXME: can we just calculate this from XInput's events? */
     if (videodata->global_mouse_changed) {
         for (i = 0; i < num_screens; i++) {
-            SDL_DisplayData *data = (SDL_DisplayData *) SDL_GetDisplayDriverData(i);
+            SDL_DisplayData *data = (SDL_DisplayData *)SDL_GetDisplayDriverData(i);
             if (data != NULL) {
                 Window root, child;
                 int rootx, rooty, winx, winy;
@@ -406,6 +436,8 @@ X11_GetGlobalMouseState(int *x, int *y)
                     buttons |= (mask & Button1Mask) ? SDL_BUTTON_LMASK : 0;
                     buttons |= (mask & Button2Mask) ? SDL_BUTTON_MMASK : 0;
                     buttons |= (mask & Button3Mask) ? SDL_BUTTON_RMASK : 0;
+                    /* Use the SDL state for the extended buttons - it's better than nothing */
+                    buttons |= (SDL_GetMouseState(NULL, NULL) & (SDL_BUTTON_X1MASK | SDL_BUTTON_X2MASK));
                     /* SDL_DisplayData->x,y point to screen origin, and adding them to mouse coordinates relative to root window doesn't do the right thing
                      * (observed on dual monitor setup with primary display being the rightmost one - mouse was offset to the right).
                      *
@@ -421,16 +453,14 @@ X11_GetGlobalMouseState(int *x, int *y)
         }
     }
 
-    SDL_assert(!videodata->global_mouse_changed);  /* The pointer wasn't on any X11 screen?! */
+    SDL_assert(!videodata->global_mouse_changed); /* The pointer wasn't on any X11 screen?! */
 
     *x = videodata->global_mouse_position.x;
     *y = videodata->global_mouse_position.y;
     return videodata->global_mouse_buttons;
 }
 
-
-void
-X11_InitMouse(_THIS)
+void X11_InitMouse(_THIS)
 {
     SDL_Mouse *mouse = SDL_GetMouse();
 
@@ -447,9 +477,18 @@ X11_InitMouse(_THIS)
     SDL_SetDefaultCursor(X11_CreateDefaultCursor());
 }
 
-void
-X11_QuitMouse(_THIS)
+void X11_QuitMouse(_THIS)
 {
+    SDL_VideoData *data = (SDL_VideoData *)_this->driverdata;
+    SDL_XInput2DeviceInfo *i;
+    SDL_XInput2DeviceInfo *next;
+
+    for (i = data->mouse_device_info; i != NULL; i = next) {
+        next = i->next;
+        SDL_free(i);
+    }
+    data->mouse_device_info = NULL;
+
     X11_DestroyEmptyCursor();
 }
 
